<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015-2016 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */
 
namespace core;

if(!defined('RS-MINI')) die();

/*
 * page v 1.0
 * 
 * Класс хранит инфу о странице
 *
 */

class page
{   
	/*
     * свойство id страницы
     *
     * @var    - integer
     * @access - public
     */
	
	public $pageID;
	
	/*
     * свойство alias страницы
     *
     * @var    - string
     * @access - public
     */
	
	public $pageAlias;
	
	/*
     * свойство имя страницы
     *
     * @var    - string
     * @access - public
     */
	
	public $pageName;
	
	/*
     * свойство узловой шаблон страницы
     *
     * @var    - string
     * @access - public
     */
	
	public $pageView;
	
	/*
     * свойство родителя страницы
     *
     * @var    - integer
     * @access - public
     */
	
	public $pageParent;
	
	/*
     * свойство тип страницы
     *
     * @var    - string
     * @access - public
     */
	
	public $pageType;
    
	/*
     * свойство хранит настоящий алиас в случае если страница динамичная
     *
     * @var    - string
     * @access - public
     */
	
	public $pageDynamicAlias;
	
	/*
     * массив с контроллерами страницы
     *
     * @var    - array
     * @access - public
     */
	
	public $controllerList;

	/*
     * контсруктор. получаем свойства страницы
     * 
     * @access - public
     * 
     * @param array page - массив с инфой о странице
     */
	
	public function __construct(&$page)
	{
		// id страницы (из таблицы rs_map)
        $this->pageID              = $page['map_id'];
		// псевдоним страницы (из таблицы rs_map или из урла
        // зависит от того статиная (из таблицы rs_map)
        // или динамичная (из урла) страница)
        $this->pageAlias           = $page['map_alias'];
        // имя страницы (из таблицы rs_map)
		$this->pageName            = $page['map_name'];
        // шаблон страницы (из таблицы rs_map)
		$this->pageView            = $page['map_view'];
        // родитель страницы (из таблицы rs_map)
		$this->pageParent          = $page['map_parent'];
        // статичная или динамичная страница (из таблицы rs_map)
		$this->pageType            = $page['map_type'];
        // реальный псевдоним страницы (сохраненный в
        // поле map_alias таблицы rs_map). Если страница статичная
        // то в ключе dynamic_alias массива $page находится NULL
		$this->pageDynamicAlias    = $page['dynamic_alias'];

		// получаем контроллеры страницы
		$this->controllerList = $this->getController();
	}
	
	/*
     * метод получаем контроллеры страницы
     * задача сформировать вот такой массив:
     *      [top]
     *          [0] => object core\controller
     *          [1] => object core\controller
     *          [2] => object core\controller
     *      [center]
     *          [0] => object core\controller
     *          [1] => object core\controller
     *          [2] => object core\controller
     *      [bottom]
     *          [0] => object core\controller
     *          [1] => object core\controller
     *          [2] => object core\controller
     * 
     * 
     * @access - private
     * @return - массив с контроллерами или false
     *
     */
	
    private function getController()
	{
        // получаем список подключенных контроллеров к станице
        $sql = ''
            . 'SELECT'
                . ' connect.connect_id           AS connect_id,'
                . ' connect.connect_value        AS connect_value,'
                . ' connect.connect_block        AS connect_block,'
                . ' connect.connect_inherit      AS connect_inherit,'
                . ' connect.connect_position     AS connect_position,'
                
                . ' connect.map_id               AS map_id,'
                
                . ' connect.controller_id        AS controller_id,'
                . ' controller.controller_name   AS controller_name'
            . ' FROM'
                . ' rs_connect                   AS connect,'
                . ' rs_controller                AS controller'
            . ' WHERE'
                . '     connect.map_id           = \'' . $this->pageID . '\''
                . ' AND connect.controller_id    = controller.controller_id'
            . ' ORDER BY connect_position'
        ;

        // если список подключений пуст
        if (!$connectList = sql::query($sql)->fetch())
            // выходим из метода
            return FALSE;
			
		// формируем массив из объектов контроллеров
        // список конроллеров
        $controllerList = array();
		// блоки к которым цепляются контроллеры
        $blocks         = array('top', 'center', 'bottom');
		
		// листаем блоки
        foreach ($blocks as $block)
			// листаем список подключений
            foreach ($connectList as $connect)
				// если блок подключения равен блоку итерации
                if ($connect['connect_block'] == $block)
					// поднимаем объект контроллера на основе данных подключения
                    $controllerList[$block][] = new controller($connect);
        
        return $controllerList;
	}
    
	/*
     * метод перезаписывает существующий контроллер вместе с параметрами
     * 
     * @access - public
     *
     * @param array inhControllerList - массив с унаследованными контроллерами
     */

    public function unionController($inhControllerList)
	{
     	// блоки к которым цепляются контроллеры
        $blocks = array('top', 'center', 'bottom');

		// если у страницы вообще нет контроллеров, то просто сохраняем наследованные контроллеры на этой страницы
		if (!$this->controllerList) {
			
			// прежде чем сохранять, необходимо контроллерам отметить, что они наследники
            // листаем блоки
			foreach ($blocks as $block)
				// если среди наследуемых контроллеров есть данный блок
                if (isset($inhControllerList[$block]))
					// листаем контроллеры этого блока
                    foreach ($inhControllerList[$block] as $inhKey => $inhController)
						// отмечаем у копии объекта контроллера что он наследуемый
                        $inhControllerList[$block][$inhKey]->inheritFlag = TRUE;
			
			// записываем список контроллеров в свойство
            $this->controllerList = $inhControllerList;
			return;
		}
		
		// если у страницы есть контроллеры
        // листаем блоки
		foreach ($blocks as $block)
			// если среди наследуемых контроллеров есть данный блок
            if (isset($inhControllerList[$block])) {
                // массив с новым порядком контроллеров
                $newList = array();
                
				// листаем список наследуемых контроллеров
                foreach ($inhControllerList[$block] as $inhKey => $inhController)
					// ищем есть ли на страницы такой же подключенный контроллеры
                    // если на странице есть листаемый блок контроллеров
					if (isset($this->controllerList[$block])) {
						
						// флаг позволяющий определить был ли найден контроллер
						$found = FALSE;
						
						// листаем контроллеры страницы
                        foreach ($this->controllerList[$block] as $controllerKey => $controller)
							// если id контроллера страницы совпадает с id контроллера наследника
                            if ($controller->controllerID == $inhController->controllerID) {
								
								// отмечаем флаг, что контроллер найден
                                $found = TRUE;
								
								// отмечаем что данный контроллер унаследован
                                $this->controllerList[$block][$controllerKey]->inheritFlag = TRUE;
		                       	
                                // запускаем метод который сольет параметры наследуемого контроллера и параметры
                                // контроллера подключенного на этой странице
								$this->controllerList[$block][$controllerKey]->unionParams($inhController->getParams());
                                
                                // записываем контроллер в новый блок
                                $newList[] = $this->controllerList[$block][$controllerKey];
							}
							
						// если контроллер не был найден
                        if ($found === FALSE) {
							// говорим что он унаследован
                            $inhController->inheritFlag = TRUE;
                            // записываем контроллер в новый блок
                            $newList[] = $inhController;
						}
							
					}
                    
                    // если на странице нет листаемого блока контроллеров
                    else {
                       	// отмечаем унаследованный контроллер как унаследованный
                        $inhController->inheritFlag = TRUE;
                        
                        // записываем контроллер в новый блок
                        $newList[]                  = $inhController;
                    }
                    
                // теперь нужно в массив $newList записать контроллеры страницы, которых
                // в массиве $newList еще нет
                // если на странице есть данный блок контроллеров
                if (isset($this->controllerList[$block]))
                    // листаем контроллеры страницы
                    foreach ($this->controllerList[$block] as $controllerKey => $controller) {
                        
                        // флаг позволяющий определить был ли найден контроллер
                        $found = FALSE;
                        
                        // листаем контроллеры лежащие в массиве $newList
                        foreach ($newList as $new)
                            // если id контроллеров совпадают
                            if ($new->controllerID == $controller->controllerID)
                                // переключаем флаг
                                $found = TRUE;
                                
                        // если контроллер среди $newList не найден
                        if ($found === FALSE)
                            // то записываем его туда
                            $newList[] = $controller;
                    }
                    
                // полностью заменяем блок контроллеров страницы
                $this->controllerList[$block] = $newList;
            }
	}
}
