<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015-2016 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */
 
namespace core;

if(!defined('RS-MINI')) die();

/*
 * view v 1.0
 * 
 * класс собирает и запускает шаблоны
 *
 */

class view
{   
    
    /*
     * массив с шаблонами
     *
     * @var    - array
     * @access - private
     */
    
    private $tplList;
    
    /*
     * путь к шаблону узла
     *
     * @var    - string
     * @access - private
     */
    
    private $tplPage;
    
    /*
     * объект дома
     *
     * @var    - object
     * @access - private
     */
    
    private $dom;
    
    /*
     * создание объекта
     * 
     * @access - public
     *
     * @param array  tplList - массив с шаблонами
     * @param string tplPage - путь к шаблону узла
     * @param object dom     - объект дома
     */
    
    public static function create($tplList, $tplPage, $dom)
    {
        // получаем имя вызываемого класса
        $who = get_called_class();
        
        // поднимает объект этого класса
        return new $who($tplList, $tplPage, $dom);
    }
    
    /*
     * контсруктор
     * 
     * @access - public
     *
     * @param array  tplList - массив с шаблонами
     * @param string tplPage - путь к шаблону узла
     * @param object dom     - объект дома
     */
    
    public function __construct($tplList, $tplPage, $dom)
    {
        // массив с шаблонами контроллера
        $this->tplList     = $tplList;
        // имя шаблона узла
        $this->tplPage     = $tplPage;
        // дом
        $this->dom         = $dom;
    }
    
    /*
     * запуск шаблонизатора
     * 
     * @access - public
     * 
     */
    
    public function run()
    {
        // выгрузка на страницу дома
        if (DOM === TRUE && !is_null(request::getHttpGet('dom')->getData())) {
            echo ''
                 .'<!DOCTYPE html>'
                 .'<html>'
                     .'<head>'
                         .'<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">'
                         .'<title>'. CMS .' v'. VERSION .' DOM</title>'
                     .'</head>'
                     .'<body>'
            ;
            
                 echo $this->getData($this->dom->getDom());
            
            echo ''
                    .'</body>'
                 .'</html>'
            ;
            
            // закрываем подключение к БД
            sql::close();
            die();
        }
        
        // или получаем страницу
        $this->getPage();
    }
    
    /*
     * получаем страницу (подключаем все нужные шаблоны)
     * 
     * @access - private
     * 
     */
    
    private function getPage()
    {
        // подготавливаем путь к шаблону кабинета
        $mainTplClass                  = '\view\\' . room::getFolder() .'\\'.'main';
        // формируем имя шаблона кабинета
        $mainName                      = room::getFolder() . '.' . 'main';
        
        // подготавливаем путь к шаблону ветки
        $rootTplClass                  = '\view\\'.(str_replace('.', '\\', $this->tplPage));
        
        // регистрируем в стеке шаблонов шаблон кабинета
        $this->tplList[$mainName]      = new $mainTplClass($this->dom);
        // регистрируем в стеке шаблонов шаблон ветки
        $this->tplList[$this->tplPage] = new $rootTplClass($this->dom);
        
        // переносим все объекты шаблонов в статичное свойство
        $this->tplList[$mainName]->setTplList($this->tplList);
        
        // закрываем подключение к БД
        sql::close();
        
        // стартуем главный шаблон
        $this->tplList[$mainName]->getTpl($mainName)->run();
    }
    
    /*
     * метод позволяет выгрузить дом на страницу (для разработчика).
     * Отладочный метод
     * 
     * @access - private
     * @return - string с html кодом
     *
     * @param array data - массив с домом
     */
    
    private function getData($data)
    {
        // строка с html кодом дерева
        $result = '';

        // если массив с узлами не пустой
        if (!empty($data))
            // листаем его
            foreach ($data as $key => $node) {
                // формируем кусочек хтмла (имя узла)
                $row = '<ul style="list-style:none; padding-left:35px;"><li>[<b>' . $node['name'] . '</b>';
                
                // если есть атрибуты
                if (!empty($node['attr']))
                    // листаем их
                    foreach ($node['attr'] as $name => $value) {
                        
                        // чистим значение атрибута от html
                        $value = strip_tags($value);

                        // если значение больше 100 симвовлов
                        if (mb_strlen($value, 'UTF-8') > 100)
                            // отризаем для показа только 100 симвовлов
                            $value = mb_substr($value, 0, 100, 'UTF-8') . '...';
                            
                        // формируем кусочек хтмл кода (имя и значение атрибута)
                        $row .= ''
                            . '<div style="padding-left:25px;">'
                                . '<span style="text-decoration:underline;">'
                                    . $name
                                . '</span>'
                                . '&nbsp;&rarr;&nbsp;'
                                . '<span style="font-style:italic;">'
                                    . $value
                                . '</span>'
                            . '</div>';
                    }
                
                // формируем кусочек хтмла
                $row .= ']';
                
                // если внутри текущего узла, есть еще узлы
                if (!empty($node['node']))
                    // перезапускаем метод с новыми данными
                    $row .= $this->getData($node['node']);

                // формируем кусочек хтмла
                $result .= $row . '[<b>/' . $node['name'] . '</b>]</li></ul>';
            }
        // если массив с узлами пустой, формируем сообщение об этом
        else
            $result = 'Нет данных';
        
        // возвращаем html
        return $result;
    }
}
